package aicare.net.cn.aibrushdemo;

import android.Manifest;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Settings;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.SeekBar;
import android.widget.Spinner;
import android.widget.TextView;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import aicare.net.cn.aibrushdemo.R;
import aicare.net.cn.aibrushdemo.DeviceDialog.OnDeviceScanListener;
import aicare.net.cn.aibrushdemo.utils.AppUtils;
import aicare.net.cn.aibrushdemo.utils.SPUtils;
import aicare.net.cn.aibrushdemo.utils.T;
import aicare.net.cn.toothbrushlibrary.bleprofile.BleProfileService;
import aicare.net.cn.toothbrushlibrary.bleprofile.BleProfileServiceReadyActivity;
import aicare.net.cn.toothbrushlibrary.entity.BatteryState;
import aicare.net.cn.toothbrushlibrary.entity.BrushDevice;
import aicare.net.cn.toothbrushlibrary.entity.GearBean;
import aicare.net.cn.toothbrushlibrary.entity.HistoryData;
import aicare.net.cn.toothbrushlibrary.entity.WorkData;
import aicare.net.cn.toothbrushlibrary.entity.WorkState;
import aicare.net.cn.toothbrushlibrary.toothbrush.ToothbrushService;
import aicare.net.cn.toothbrushlibrary.utils.ElectricityUtils;
import aicare.net.cn.toothbrushlibrary.utils.FileUtils;
import aicare.net.cn.toothbrushlibrary.utils.L;
import aicare.net.cn.toothbrushlibrary.utils.ParseData;
import aicare.net.cn.toothbrushlibrary.utils.ToothbrushBleConfig;

public class MainActivity extends BleProfileServiceReadyActivity implements View.OnClickListener,
        OnDeviceScanListener {

    private final static String TAG = "MainActivity";

    private final static String PAIR_INFO = "pair_info";
    private long timeMills;

    private Button btn_connect_state, btn_set_time, btn_set_sn, btn_clear_log, btn_set_did,
            btn_set_name, btn_get_mode;
    private Button btn_query_work_state, btn_query_battery_state, btn_query_time,
            btn_query_device_sn, btn_set_did_two, btn_get_did, btn_get_version;
    private Button btn_on_off, btn_query_history, btn_cancel_history, btn_delete_history;
    private TextView tv_duration;
    private SeekBar seek_bar_duration;
    private Spinner spinner_mode, spinner_freq_level, spinner_duty_level;
    private Button btn_set_work_param;
    private ListView lv_show_info;
    private ArrayAdapter listAdapter;
    private EditText et_device_sn, et_did, et_name, et_manufacturer_id, et_model_id;
    private BrushDevice mBrushDevice;
    private List<String> infoList;
    private String[] modeArr, freqLevelArr, dutyLevelArr, durationArr, strengthArr, workStateArr,
            setStateArr, chargerArr, batteryArr, powerArr, workNodeArr, authState;

    private ToothbrushService.ToothbrushBinder binder;

    private static WorkState mWorkState;
    private final static int DEFAULT_DURATION = 6;
    private final static int DEFAULT_MODE = ToothbrushBleConfig.CLEAN_MODE;
    private final static int DEFAULT_FREQ_LEVEL = 0;
    private final static int DEFAULT_DUTY_LEVEL = 0;
    private final static int DEFAULT_PRESET_ID = 0;

    //2016-10-14
    private LinearLayout ll_tv_freq_duty, ll_spinner_freq_duty;
    private TextView tv_preset_id;
    private Spinner spinner_preset_id;
    private String[] presetIdArr;
    private Button btn_update;

    private ProgressDialog progressDialog;
    private aicare.net.cn.aibrushdemo.DeviceDialog devicesDialog;

    private String version;

    private Button btn_connect;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        L.isDebug = true;
        setContentView(R.layout.activity_main);
        initData();
        initViews();
        initDialogs();
        initEvents();
        initDefault();
    }


    /**
     * 初始化默认选项
     */
    private void initDefault() {
        seek_bar_duration.setProgress(DEFAULT_DURATION);
        spinner_mode.setSelection(DEFAULT_MODE - 1);
        spinner_freq_level.setSelection(DEFAULT_FREQ_LEVEL);
        spinner_duty_level.setSelection(DEFAULT_DUTY_LEVEL);
        spinner_preset_id.setSelection(DEFAULT_PRESET_ID);
    }

    private void initData() {
        modeArr = getResources().getStringArray(R.array.arr_mode);
        freqLevelArr = getResources().getStringArray(R.array.arr_freq_level);
        dutyLevelArr = getResources().getStringArray(R.array.arr_duty_level);
        durationArr = getResources().getStringArray(R.array.arr_duration);
        workStateArr = getResources().getStringArray(R.array.arr_work_state);
        setStateArr = getResources().getStringArray(R.array.arr_set_state);
        chargerArr = getResources().getStringArray(R.array.arr_charger);
        batteryArr = getResources().getStringArray(R.array.arr_battery);
        powerArr = getResources().getStringArray(R.array.arr_power);
        strengthArr = getResources().getStringArray(R.array.arr_strength);
        workNodeArr = getResources().getStringArray(R.array.arr_work_node);
        authState = getResources().getStringArray(R.array.arr_auth_state);
        presetIdArr = new String[]{"1", "2", "3", "4", "5"};
        infoList = new ArrayList<>();
        mWorkState = new WorkState((byte) 0x00, (byte) 0x00, (byte) 0x00);

        timeMills = (long) SPUtils.get(this, PAIR_INFO, 0L);
    }

    private void initViews() {
        btn_connect_state = (Button) findViewById(R.id.btn_connect_state);
        btn_query_work_state = (Button) findViewById(R.id.btn_query_work_state);
        btn_query_battery_state = (Button) findViewById(R.id.btn_query_battery_state);
        btn_query_time = (Button) findViewById(R.id.btn_query_time);
        btn_query_device_sn = (Button) findViewById(R.id.btn_query_device_sn);
        btn_on_off = (Button) findViewById(R.id.btn_on_off);
        btn_query_history = (Button) findViewById(R.id.btn_query_history);
        btn_cancel_history = (Button) findViewById(R.id.btn_cancel_history);
        btn_delete_history = (Button) findViewById(R.id.btn_delete_history);
        btn_set_time = (Button) findViewById(R.id.btn_set_time);
        btn_set_sn = (Button) findViewById(R.id.btn_set_sn);
        btn_set_did = (Button) findViewById(R.id.btn_set_did);
        btn_set_name = (Button) findViewById(R.id.btn_set_name);
        btn_get_mode = (Button) findViewById(R.id.btn_get_mode);
        btn_set_did_two = (Button) findViewById(R.id.btn_set_did_two);
        btn_get_did = (Button) findViewById(R.id.btn_get_did);
        btn_get_version = (Button) findViewById(R.id.btn_get_version);

        et_device_sn = (EditText) findViewById(R.id.et_device_sn);
        et_did = (EditText) findViewById(R.id.et_did);
        et_name = (EditText) findViewById(R.id.et_name);
        et_manufacturer_id = (EditText) findViewById(R.id.et_manufacturer_id);
        et_model_id = (EditText) findViewById(R.id.et_model_id);
        tv_duration = (TextView) findViewById(R.id.tv_duration);
        seek_bar_duration = (SeekBar) findViewById(R.id.seek_bar_duration);

        btn_clear_log = (Button) findViewById(R.id.btn_clear_log);

        spinner_mode = (Spinner) findViewById(R.id.spinner_mode);
        spinner_mode.setAdapter(new ArrayAdapter<>(this, android.R.layout.simple_spinner_item,
                modeArr));

        spinner_freq_level = (Spinner) findViewById(R.id.spinner_freq_level);
        spinner_freq_level.setAdapter(new ArrayAdapter<>(this,
                android.R.layout.simple_spinner_item, freqLevelArr));

        spinner_duty_level = (Spinner) findViewById(R.id.spinner_duty_level);
        spinner_duty_level.setAdapter(new ArrayAdapter<>(this,
                android.R.layout.simple_spinner_item, dutyLevelArr));

        btn_set_work_param = (Button) findViewById(R.id.btn_set_work_param);
        lv_show_info = (ListView) findViewById(R.id.lv_show_info);
        listAdapter = new ArrayAdapter<>(this, android.R.layout.simple_list_item_1, infoList);
        lv_show_info.setAdapter(listAdapter);

        //2016-10-14
        ll_tv_freq_duty = (LinearLayout) findViewById(R.id.ll_tv_freq_duty);
        ll_spinner_freq_duty = (LinearLayout) findViewById(R.id.ll_spinner_freq_duty);
        tv_preset_id = (TextView) findViewById(R.id.tv_preset_id);
        spinner_preset_id = (Spinner) findViewById(R.id.spinner_presett_id);
        spinner_preset_id.setAdapter(new ArrayAdapter<>(this,
                android.R.layout.simple_spinner_item, presetIdArr));
        btn_update = (Button) findViewById(R.id.btn_update);

    }

    private void initEvents() {
        btn_connect_state.setOnClickListener(this);
        btn_query_work_state.setOnClickListener(this);
        btn_query_battery_state.setOnClickListener(this);
        btn_query_time.setOnClickListener(this);
        btn_query_device_sn.setOnClickListener(this);
        btn_set_work_param.setOnClickListener(this);
        btn_on_off.setOnClickListener(this);
        btn_query_history.setOnClickListener(this);
        btn_cancel_history.setOnClickListener(this);
        btn_delete_history.setOnClickListener(this);
        btn_set_time.setOnClickListener(this);
        btn_set_sn.setOnClickListener(this);
        btn_clear_log.setOnClickListener(this);
        btn_set_did.setOnClickListener(this);
        btn_set_did_two.setOnClickListener(this);
        btn_set_name.setOnClickListener(this);
        btn_get_mode.setOnClickListener(this);
        btn_get_did.setOnClickListener(this);
        btn_get_version.setOnClickListener(this);

        spinner_mode.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                switch (position) {
                    case 0:
                        mWorkState.setMode(ToothbrushBleConfig.CLEAN_MODE);
                        setDefault(false, false);
                        break;
                    case 1:
                        mWorkState.setMode(ToothbrushBleConfig.WHITE_MODE);
                        setDefault(false, false);
                        break;
                    case 2:
                        mWorkState.setMode(ToothbrushBleConfig.MASSAGE_MODE);
                        setDefault(false, false);
                        break;
                    case 3:
                        mWorkState.setMode(ToothbrushBleConfig.SENSITIVE_MODE);
                        setDefault(false, false);
                        break;
                    case 4:
                        mWorkState.setMode(ToothbrushBleConfig.PRESET_MODE);
                        setDefault(true, true);
                        break;
                    case 5:
                        mWorkState.setMode(ToothbrushBleConfig.CUSTOM_MODE);
                        setDefault(false, true);
                        break;
                    case 6:
                        mWorkState.setMode(ToothbrushBleConfig.POLISH_MODE);
                        mWorkState.setPresetId((byte) 1);
                        setDefault(false, false);
                        break;
                    case 7:
                        mWorkState.setMode(ToothbrushBleConfig.GUM_CARE_MODE);
                        mWorkState.setPresetId((byte) 2);
                        setDefault(false, false);
                        break;
                    case 8:
                        mWorkState.setMode(ToothbrushBleConfig.REFRESH_MODE);
                        mWorkState.setPresetId((byte) 3);
                        setDefault(false, false);
                        break;
                    case 9:
                        mWorkState.setMode(ToothbrushBleConfig.MASCARE_MODE);
                        mWorkState.setPresetId((byte) 4);
                        setDefault(false, false);
                        break;
                    case 10:
                        mWorkState.setMode(ToothbrushBleConfig.SOFT_MODE);
                        mWorkState.setPresetId((byte) 5);
                        setDefault(false, false);
                        break;
                }
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        spinner_freq_level.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                mWorkState.setFreqLevel((byte) position);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        spinner_duty_level.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                mWorkState.setDutyLevel((byte) position);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        seek_bar_duration.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
            @Override
            public void onProgressChanged(SeekBar seekBar, int progress, boolean fromUser) {
                mWorkState.setDuration(ParseData.int2Bytes(progress + 1)[3]);
                setDuration(progress);
            }

            @Override
            public void onStartTrackingTouch(SeekBar seekBar) {

            }

            @Override
            public void onStopTrackingTouch(SeekBar seekBar) {

            }
        });

        //2016-10-14
        spinner_preset_id.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                mWorkState.setPresetId((byte) (position + 1));
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
        btn_update.setOnClickListener(this);
    }

    private void initDialogs() {
        initProgressDialog();
        initDevicesDialog();
    }

    private void initProgressDialog() {
        progressDialog = new ProgressDialog(this);
        progressDialog.setTitle(R.string.update_ing);
        progressDialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
        progressDialog.setIndeterminate(false);
        progressDialog.setCancelable(false);
    }

    private void initDevicesDialog() {
        devicesDialog = new aicare.net.cn.aibrushdemo.DeviceDialog(this, this);
    }

    private void setDuration(int duration) {
        tv_duration.setText(String.format(getResources().getString(R.string.work_duration),
                durationArr[duration]));
    }

    /**
     * 非自定义模式下不能设置频率和占空比档位以及持续时间
     * 所有状态重置为默认
     */
    private void setDefault(boolean isPresetMode, boolean isCustomMode) {
        if (isCustomMode) {
            spinner_freq_level.setSelection(1);
            spinner_duty_level.setSelection(1);
        } else {
            spinner_freq_level.setSelection(0);
            spinner_duty_level.setSelection(0);
        }
        spinner_freq_level.setEnabled(isCustomMode);
        spinner_duty_level.setEnabled(isCustomMode);
        if (isPresetMode) {
            ll_spinner_freq_duty.setVisibility(View.GONE);
            ll_tv_freq_duty.setVisibility(View.GONE);
            tv_preset_id.setVisibility(View.VISIBLE);
            spinner_preset_id.setVisibility(View.VISIBLE);
        } else {
            ll_spinner_freq_duty.setVisibility(View.VISIBLE);
            ll_tv_freq_duty.setVisibility(View.VISIBLE);
            tv_preset_id.setVisibility(View.GONE);
            spinner_preset_id.setVisibility(View.GONE);
        }
    }


    @Override
    protected void onServiceBinded(ToothbrushService.ToothbrushBinder binder) {
        this.binder = binder;
    }

    @Override
    protected void onServiceUnbinded() {

    }

    @Override
    public void onStateChanged(String address, int state) {
        super.onStateChanged(address, state);
        switch (state) {
            case BleProfileService.STATE_CONNECTED:
                L.e(TAG, "STATE_CONNECTED");
                btn_connect_state.setText(String.format(getResources().getString(R.string.connected_device), binder.getDeviceAddress()));
                break;
            case BleProfileService.STATE_DISCONNECTED:
                L.e(TAG, "STATE_DISCONNECTED");
                btn_connect_state.setText(R.string.not_connect_device);
                break;
            case BleProfileService.STATE_SERVICES_DISCOVERED:
                L.e(TAG, "STATE_SERVICES_DISCOVERED");
                break;
            case BleProfileService.STATE_INDICATION_SUCCESS:
                L.e(TAG, "STATE_INDICATION_SUCCESS");
                if (isDeviceConnected()) {
                    if (timeMills == 0) {
                        timeMills = System.currentTimeMillis();
                        SPUtils.put(this, PAIR_INFO, timeMills);
                    }
                    binder.requestPair(timeMills);
                }
                break;
        }
    }

    @Override
    public void onError(final String msg, final int errorCode) {
        T.showLong(MainActivity.this, "Message: " + msg + " errorCode = " + errorCode);
    }

    @Override
    public void getWorkState(WorkState workState) {
        L.i(MainActivity.class, workState.toString());
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append(getStr(R.string.work_state));
        stringBuffer.append(workStateArr[workState.getWorkState()]);
        stringBuffer.append(getStr(R.string.work_state_mode));
        stringBuffer.append(modeArr[workState.getMode() - 1]);
        if (workState.getMode() == ToothbrushBleConfig.PRESET_MODE) {
            stringBuffer.append(getStr(R.string.work_data_preset_id)).append(ParseData.getData(workState.getPresetId()));
        } else {
            stringBuffer.append(getStr(R.string.work_state_feq));
            stringBuffer.append(freqLevelArr[workState.getFreqLevel()]);
            stringBuffer.append(getStr(R.string.work_state_duty));
            stringBuffer.append(dutyLevelArr[workState.getDutyLevel()]);
        }
        stringBuffer.append(getStr(R.string.work_state_duration));
        stringBuffer.append(durationArr[workState.getDuration() - 1]);
        stringBuffer.append(getStr(R.string.work_state_efforts));
        stringBuffer.append(strengthArr[workState.getStrength()]);
        stringBuffer.append(getStr(R.string.work_state_node));
        stringBuffer.append(workNodeArr[workState.getWorkNode()]);
        showInfo(stringBuffer.toString());
    }

    @Override
    public void getBatteryState(BatteryState batteryState) {
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append(getStr(R.string.power_status));
        stringBuffer.append(chargerArr[batteryState.getChargerState() - 1]);
        stringBuffer.append(getStr(R.string.battery_status));
        stringBuffer.append(batteryArr[batteryState.getBatteryState()]);
        stringBuffer.append(getStr(R.string.charger_status));
        stringBuffer.append(powerArr[batteryState.getPowerState()]);
        stringBuffer.append(getStr(R.string.current_voltage));
        stringBuffer.append(batteryState.getBattery() + " V");
        stringBuffer.append(";"+ ElectricityUtils.getPowerPercent(batteryState.getBattery())+"%");
        showInfo(stringBuffer.toString());
    }

    @Override
    public void onResultChange(int type, final String result) {
        switch (type) {
            case ToothbrushService.RESULT_TYPE_SN:
                showInfo(getStr(R.string.device_id) + result);
                break;
            case ToothbrushService.RESULT_TYPE_TIME:
                showInfo(getStr(R.string.device_time) + result);
                break;
            case ToothbrushService.RESULT_TYPE_UNKNOWN:
                showInfo(getStr(R.string.unknown_command) + result);
                break;
            case ToothbrushService.RESULT_TYPE_OTA:
                isError = true;
                showErrorDialog(result);
                if (progressDialog != null) {
                    progressDialog.dismiss();
                }
                break;
        }
    }

    @Override
    public void getResultState(int type, byte state) {
        if (state == ToothbrushBleConfig.UNKNOWN_COMMAND) {
            showInfo(getResources().getString(R.string.unknown_command));
            return;
        }
        switch (type) {
            case ToothbrushBleConfig.SET_PARAM_STATE:
                showInfo(getStr(R.string.set_param_status) + setStateArr[state - 1]);
                break;
            case ToothbrushBleConfig.SET_TIME_STATE:
                showInfo(getStr(R.string.set_time_status) + setStateArr[state - 1]);
                break;
            case ToothbrushBleConfig.SET_SN_STATE:
                showInfo(getStr(R.string.set_id_status) + setStateArr[state - 1]);
                break;
            case ToothbrushBleConfig.HISTORY_CANCEL:
                showInfo(getStr(R.string.cancel_history_status) + setStateArr[state - 1]);
                break;
            case ToothbrushBleConfig.HISTORY_DELETE:
                showInfo(getStr(R.string.delete_history_status) + setStateArr[state - 1]);
                break;
            case ToothbrushBleConfig.DID:
                showInfo(getStr(R.string.set_did_state) + setStateArr[state]);
                break;
            case ToothbrushBleConfig.DEVICE_NAME:
                showInfo(getStr(R.string.set_name_state) + setStateArr[state]);
                break;
        }
    }

    private int progress = 0;
    private int count = 0;

    @Override
    public void getHistoryCount(int count) {
        showInfo(getStr(R.string.history_count) + count);
        this.count = count;
    }

    @Override
    public void getHistoryData(HistoryData historyData) {
        WorkData workData = historyData.getWorkData();
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append(getStr(R.string.history));
        stringBuffer.append(getStr(R.string.history_id));
        stringBuffer.append(historyData.getId());
        stringBuffer.append(getStr(R.string.history_mode));
        setWorkDataStr(workData, stringBuffer);
        showInfo(stringBuffer.toString());
        progress = (historyData.getId() + 1) * 100 / count;
        L.e(TAG, "progress = " + progress);
    }

    private void setWorkDataStr(WorkData workData, StringBuffer stringBuffer) {
        int modeIndex = workData.getMode() - 1;
        if (modeIndex > modeArr.length) {
            modeIndex = modeArr.length - 1;
        }

        stringBuffer.append(modeArr[modeIndex]);
        if (workData.getMode() == ToothbrushBleConfig.PRESET_MODE) {
            stringBuffer.append(getStr(R.string.work_data_preset_id)).append(ParseData.getData(workData.getPresetId()));
        } else {
            stringBuffer.append(getStr(R.string.work_data_feq));
            stringBuffer.append(freqLevelArr[workData.getFreqLevel()]);
            stringBuffer.append(getStr(R.string.work_data_duty));
            stringBuffer.append(dutyLevelArr[workData.getDutyLevel()]);
        }
        stringBuffer.append(getStr(R.string.work_data_duration));
        stringBuffer.append(durationArr[workData.getDuration() - 1]);
        stringBuffer.append(getStr(R.string.work_data_start_time));
        stringBuffer.append(workData.getStartTime());
        stringBuffer.append(getStr(R.string.work_data_work_time));
        stringBuffer.append(workData.getWorkTime() + "s");
        stringBuffer.append(getStr(R.string.work_data_intensity_count));
        stringBuffer.append(workData.getOverPowerCount());
        stringBuffer.append(getStr(R.string.work_data_intensity_time));
        stringBuffer.append(workData.getOverPowerTime() + "s");
        stringBuffer.append(getStr(R.string.work_data_left_time));
        stringBuffer.append(workData.getLeftBrushTime() + "s");
        stringBuffer.append(getStr(R.string.work_data_right_time));
        stringBuffer.append(workData.getRightBrushTime() + "s");
        stringBuffer.append("]");
    }

    @Override
    public void getWorkData(WorkData workData) {
        StringBuffer stringBuffer = new StringBuffer();
        stringBuffer.append(getStr(R.string.work_data));
        stringBuffer.append(getStr(R.string.work_data_mode));
        setWorkDataStr(workData, stringBuffer);
        showInfo(stringBuffer.toString());
    }

    @Override
    public void getAuthState(byte state) {
        showInfo(getStr(R.string.auth_state) + authState[state - 1]);
    }

    @Override
    protected void getVersion(String version) {
        this.version = version;
        showInfo(version);
    }

    @Override
    protected void getGears(GearBean gearBean) {
        //档位信息
        showInfo(Arrays.toString(gearBean.getGears()[0]) + "\n" + Arrays.toString(gearBean.getGears()[1]));
    }

    @Override
    public void onOtaSuccess() {
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
        showOtaSuccessDialog();
    }

    private boolean isError;

    @Override
    public void onProgressChange(final float progress) {
        Log.e(TAG, "progress = " + progress);
        if (!isError) {
            progressDialog.show();
            progressDialog.setProgress((int) progress);
        }
    }

    @Override
    protected void getDevice(final BrushDevice device) {
        runOnUiThread(new Runnable() {
            @Override
            public void run() {
                devicesDialog.setDevice(device);
                Log.e(TAG, "version = " + version);
            }
        });
    }

    private String getStr(int resId) {
        return getResources().getString(resId);
    }

    private void showInfo(String str) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(getCurrentTime());
        stringBuilder.append("\n");
        stringBuilder.append(str);
        infoList.add(stringBuilder.toString());
        listAdapter.notifyDataSetChanged();
        lv_show_info.setSelection(infoList.size() - 1);
    }


    /**
     * 获取当前系统时间
     *
     * @return
     */
    public String getCurrentTime() {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date curDate = new Date(System.currentTimeMillis());
        String str = formatter.format(curDate);
        return str;
    }

    @Override
    protected void onDestroy() {
        if (isDeviceConnected()) {
            binder.disconnect();
        }
        super.onDestroy();
        //stopService(new Intent(this, ToothbrushService.class));
        stopScan();
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btn_connect_state:
                if (isDeviceConnected()) {
                    binder.disconnect();
                } else {
                    initPermissions();

                }
                break;
            case R.id.btn_query_work_state:
                if (isDeviceConnected()) {
                    binder.queryWorkState();
                }
                break;
            case R.id.btn_query_battery_state:
                if (isDeviceConnected()) {
                    binder.queryBatteryState();
                }
                break;
            case R.id.btn_set_work_param:
                if (mWorkState.getMode() == ToothbrushBleConfig.CUSTOM_MODE && (mWorkState.getFreqLevel() == (byte) 0x00 || mWorkState.getDutyLevel() == (byte) 0x00)) {
                    T.showLong(this, R.string.custom_tips);
                } else {
                    if (isDeviceConnected() && mWorkState != null) {
                        binder.setWorkParam(mWorkState);
                    }
                }
                break;
            case R.id.btn_on_off:
                if (isDeviceConnected()) {
                    binder.changeWorkState();
                }
                break;

            case R.id.btn_query_time:
                if (isDeviceConnected()) {
                    binder.queryTime();
                }
                break;
            case R.id.btn_query_device_sn:
                if (isDeviceConnected()) {
                    binder.queryDeviceSN();
                }
                break;
            case R.id.btn_query_history:
                if (isDeviceConnected()) {
                    binder.operateHistory(ToothbrushBleConfig.QUERY_HISTORY);
                }
                break;
            case R.id.btn_cancel_history:
                if (isDeviceConnected()) {
                    binder.operateHistory(ToothbrushBleConfig.CANCEL_HISTORY);
                }
                break;
            case R.id.btn_delete_history:
                if (isDeviceConnected()) {
                    binder.operateHistory(ToothbrushBleConfig.DELETE_HISTORY);
                }
                break;
            case R.id.btn_set_time:
                if (isDeviceConnected()) {
                    binder.setTime();
                }
                break;
            case R.id.btn_set_sn:
                if (isDeviceConnected()) {
                    String sn = et_device_sn.getText().toString().trim();
                    if (TextUtils.isEmpty(sn)) {
                        T.showShort(this, R.string.device_id_null);
                    } else {
                        if (ParseData.isDeviceSN(sn)) {
                            binder.setDeviceSN(sn);
                        } else {
                            T.showShort(this, R.string.device_id_format);
                        }
                    }
                }
                break;
            case R.id.btn_clear_log:
                infoList.clear();
                listAdapter.notifyDataSetChanged();
                break;
            case R.id.btn_update:
                if (isDeviceConnected()) {
                    showSelectFileDialog();
                }
                break;

            case R.id.btn_set_did:
                if (isDeviceConnected()) {
                    String did = et_did.getText().toString().trim();
                    if (TextUtils.isEmpty(did)) {
                        T.showShort(this, R.string.device_id_null);
                    } else {
                        binder.setDid(Integer.valueOf(did));
                    }
                }
                break;
            case R.id.btn_set_name:
                if (isDeviceConnected()) {
                    String name = et_name.getText().toString();
                    if (name.getBytes().length > 16) {
                        T.showShort(this, R.string.name_too_long);
                        return;
                    }
                    binder.setName(name);
                }
                break;

            case R.id.btn_get_mode:
                if (isDeviceConnected()) {
                    binder.queryGear();
                }
                break;
            case R.id.btn_set_did_two:
                String manufacturerId = et_manufacturer_id.getText().toString().trim();
                String modelId = et_model_id.getText().toString().trim();
                if (TextUtils.isEmpty(manufacturerId) || TextUtils.isEmpty(modelId)) {
                    T.showShort(this, R.string.did_null);
                } else {
                    int manufacturerIdInt = Integer.valueOf(manufacturerId);
                    int modelIdInt = Integer.valueOf(modelId);
                    if (manufacturerIdInt < 2 || manufacturerIdInt > 1022) {
                        T.showShort(this, R.string.manufacturer_id_error);
                        return;
                    }
                    if (modelIdInt < 1 || modelIdInt > 62) {
                        T.showShort(this, R.string.model_id_error);
                        return;
                    }
                    if (isDeviceConnected()) {
                        binder.setDid(ToothbrushBleConfig.getDid(manufacturerIdInt, modelIdInt));
                    }
                    et_did.setText(String.valueOf(ToothbrushBleConfig.getDid(manufacturerIdInt,
                            modelIdInt)));
                }

                break;
            case R.id.btn_get_did:
                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.append(getCurrentTime());
                stringBuilder.append("\n");
                stringBuilder.append("DID:" + mBrushDevice.getDID());
                infoList.add(stringBuilder.toString());
                listAdapter.notifyDataSetChanged();
                lv_show_info.setSelection(infoList.size() - 1);
                break;

            case R.id.btn_get_version:
                if (isDeviceConnected() && !TextUtils.isEmpty(version)) {
                    getVersion(version);
                }
                break;

        }
    }

    private void showErrorDialog(String error) {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.update_error);
        builder.setMessage(error);
        builder.setCancelable(false);
        builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                isError = false;
            }
        });
        builder.create().show();
    }

    private void showSelectFileDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(String.format(getResources().getString(R.string.select_file),
                this.version));
        if (FileUtils.list().size() == 0) {
            builder.setMessage(R.string.no_file);
        } else {
            final String[] fileNames = FileUtils.list().toArray(new String[]{});
            final SingleChoiceListener listener = new SingleChoiceListener();
            builder.setSingleChoiceItems(fileNames, 0, listener);
            builder.setPositiveButton(R.string.ok, new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialog, int which) {
                    String fileName = fileNames[listener.getWhich()];
                    //if (ParseData.compareVersion(fileName, version)) {
                    if (isDeviceConnected()) {
                        binder.startUpdate(fileName);
                    } else {
                        T.showShort(MainActivity.this, R.string.disconnected);
                    }
                    /*} else {
                        T.showShort(MainActivity.this, R.string.select_right_file);
                    }*/
                }
            });
        }
        builder.setNegativeButton(R.string.cancel, null);

        builder.create().show();
    }

    @Override
    public void scan() {
        startScan();
        devicesDialog.setScanning(isScanning());
    }

    /**
     * 初始化请求权限
     */
    private void initPermissions() {
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.ACCESS_COARSE_LOCATION) == PackageManager.PERMISSION_GRANTED) {
            devicesDialog.show();
            devicesDialog.startScanDevice();

        } else {
            //没有权限
            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.ACCESS_COARSE_LOCATION}, 1);
        }

    }


    private void initPermissionsServer() {
        if (AppUtils.isLocServiceEnable(this)) {
            devicesDialog.show();
            devicesDialog.startScanDevice();
        } else {
            new android.support.v7.app.AlertDialog.Builder(this).setTitle(this.getString(R.string.hint)).setMessage(this.getString(R.string.permissions_server)).setPositiveButton(this.getString(R.string.ok), new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialog, int which) {
                    //引导用户至设置页手动授权
                    AppUtils.startLocationActivity(MainActivity.this, 1);

                }
            }).setNegativeButton(getString(R.string.cancel), new DialogInterface.OnClickListener() {
                @Override
                public void onClick(DialogInterface dialog, int which) {
                    if (dialog != null) {
                        dialog.cancel();
                    }
                }
            }).show();

        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions,
                                           @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode != 1) {
            return;
        }
        if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
            initPermissionsServer();//定位服务判断
        } else {

            if (ActivityCompat.shouldShowRequestPermissionRationale(this, permissions[0])) {
                //权限请求失败，但未选中“不再提示”选项
                new android.support.v7.app.AlertDialog.Builder(this).setTitle(this.getString(R.string.hint)).setMessage(this.getString(R.string.permissions)).setPositiveButton(this.getString(R.string.ok), new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        //引导用户至设置页手动授权
                        Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                        Uri uri = Uri.fromParts("package",
                                getApplicationContext().getPackageName(), null);
                        intent.setData(uri);
                        startActivity(intent);
                    }
                }).setNegativeButton(getString(R.string.cancel),
                        new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        if (dialog != null) {
                            dialog.cancel();
                        }
                    }
                }).show();
            } else {
                //权限请求失败，选中“不再提示”选项
//                T.showShort(MainActivity.this, "获取权限失败");
                new android.support.v7.app.AlertDialog.Builder(this).setTitle(this.getString(R.string.hint)).setMessage(this.getString(R.string.permissions)).setPositiveButton(this.getString(R.string.ok), new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        //引导用户至设置页手动授权
                        Intent intent = new Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                        Uri uri = Uri.fromParts("package",
                                getApplicationContext().getPackageName(), null);
                        intent.setData(uri);
                        startActivity(intent);

                    }
                }).setNegativeButton(getString(R.string.cancel),
                        new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        if (dialog != null) {
                            dialog.cancel();
                        }
                    }
                }).show();
            }

        }

    }

    @Override
    public void stop() {
        stopScan();
        devicesDialog.setScanning(isScanning());
    }

    @Override
    public void connect(BrushDevice brushDevice) {
        mBrushDevice = brushDevice;
        startConnectService(brushDevice.getAddress());
    }


    private class SingleChoiceListener implements DialogInterface.OnClickListener {

        private int which;

        @Override
        public void onClick(DialogInterface dialog, int which) {
            this.which = which;
        }

        public int getWhich() {
            return which;
        }

    }

    private void showOtaSuccessDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(R.string.update_success);
        builder.setMessage(R.string.reboot_now);
        builder.setCancelable(false);
        builder.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                if (isDeviceConnected()) {
                    binder.reboot();
                }
            }
        });
        builder.setNegativeButton(R.string.no, null);
        builder.create().show();
    }
}
